<?php

namespace Modules\Cadastros\Services;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Util\SupUtilities;

class FornecedoresParaComprasExtrasService
{
    protected $connection;
    
    public function __construct($entityManager, $environment, SupUtilities $supUtilities) {
        $this->connection = $entityManager->getConnection();
        $this->environment = $environment;
        $this->supUtilities = $supUtilities;
    }
      
    public function findFornecedor($row) {
        $query = 'SELECT FORNECMPEXTRA.CDPAIS, FORNECMPEXTRA.SGESTADO, FORNECMPEXTRA.CDMUNICIPIO, FORNECMPEXTRA.CDFORNECED, FORNECEDOR.IDTPIJURFORN, FORMATMASK(FORNECEDOR.IDTPIJURFORN, FORNECEDOR.NRINSJURFORN) NRINSJURFORN, FORNECEDOR.NMRAZSOCFORN, FORNECEDOR.NMFANTFORN, FORNECEDOR.NRORG FROM FORNECMPEXTRA, FORNECEDOR WHERE FORNECMPEXTRA.CDPAIS = :CDPAIS AND FORNECMPEXTRA.SGESTADO = :SGESTADO AND FORNECMPEXTRA.CDMUNICIPIO = :CDMUNICIPIO AND FORNECMPEXTRA.CDFORNECED = :CDFORNECED ORDER BY NRINSJURFORN';
        $params = [
            'CDPAIS'  => $row['CDPAIS'],
            'SGESTADO'  => $row['SGESTADO'],
            'CDMUNICIPIO'  => $row['CDMUNICIPIO'],
            'CDFORNECED'  => $row['CDFORNECED']
        ];
        return  $this->supUtilities->crud($query, $params);
    }
    
    public function verifyData($data, $parameter = "", $country = ""){
        //Todos fornecedores
        if($data === "todosFornecedores"){
            $query = 'SELECT CDFORNECED, NRINSJURFORN FROM FORNECEDOR';
            return  $this->supUtilities->crud($query);
        }
        
        //Todos municipios
        if($data === "todosMunicipios"){
            $query = 'SELECT CDPAIS, CDMUNICIPIO, SGESTADO FROM MUNICIPIO WHERE CDPAIS = :CDPAIS';
            $params = ['CDPAIS' => $country];
            return  $this->supUtilities->crud($query, $params);
        }

        //Fornecedores filtrados para exportação de fornecedores
        if($data === "fornecedoresExportados"){
            $query = "SELECT CDFORNECED FROM FORNECEDOR WHERE CDFORNECED <> :P_CDFORNECED ORDER BY IDTPIJURFORN, NRINSJURFORN";
            $params = ['P_CDFORNECED'  => $parameter];
            return  $this->supUtilities->crud($query,$params);
        }
        
        //Municipios atendidos para exportação de fornecedores
        if($data === "municipiosAtendidos"){
            $query = 'SELECT FORNECMPEXTRA.CDPAIS, FORNECMPEXTRA.SGESTADO, FORNECMPEXTRA.CDMUNICIPIO, FROM FORNECMPEXTRA, MUNICIPIO WHERE FORNECMPEXTRA.CDFORNECED = :P_CDFORNECED ORDER BY FORNECMPEXTRA.CDPAIS, FORNECMPEXTRA.SGESTADO, FORNECMPEXTRA.CDMUNICIPIO';
            $params = ['P_CDFORNECED'  => $parameter];
            return  $this->supUtilities->crud($query,$params);
        }
        
    }
    
    public function insertFornecedor($row){ 
        try {
            $query = "INSERT INTO FORNECMPEXTRA (CDFORNECED, CDPAIS, SGESTADO, CDMUNICIPIO) VALUES (:CDFORNECED, :CDPAIS, :SGESTADO, :CDMUNICIPIO)";
            $params = [
                'CDFORNECED' => $row['CDFORNECED'],
                'CDPAIS'   => $row['CDPAIS'],
                'SGESTADO'      => $row['SGESTADO'],
                'CDMUNICIPIO'      => $row['CDMUNICIPIO']
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao tentar salvar fornecedor!');
        }
    }
    
    public function deleteFornecedor($row){ 
        try {
            $query = "DELETE FROM FORNECMPEXTRA WHERE CDFORNECED = :CDFORNECED AND CDPAIS  = :CDPAIS AND SGESTADO = :SGESTADO  AND CDMUNICIPIO = :CDMUNICIPIO";
            $params = [
                'CDFORNECED' => $row['CDFORNECED'],
                'CDPAIS'   => $row['CDPAIS'],
                'SGESTADO'      => $row['SGESTADO'],
                'CDMUNICIPIO'      => $row['CDMUNICIPIO']
            ];
            $this->supUtilities->crud($query, $params);
        }
        catch (Exception $e) {
            throw new \Exception('Erro ao tentar remover fornecedor!');
        }
    }

    public function cadastramentoAutomatico($row){ 
        $tipoCadastro = $row['TIPOCADASTRO'];
        $dadosMunicipio = $row['CDMUNICIPIO'];
        $dadosFornecedor = $row['NRINSJURFORN'];
        
        foreach($dadosMunicipio as $key => $value){
            $municipio = $value['CDMUNICIPIO'];
            $estado = $value['SGESTADO'];
            $pais = $value['CDPAIS'];
            foreach($dadosFornecedor as $index => $data){
                $forneced = $data['CDFORNECED'];
                if($tipoCadastro === "I"){
                    try {
                        $this->supUtilities->iniciaTransacao();
                        $query = "INSERT INTO FORNECMPEXTRA (CDFORNECED, CDPAIS, SGESTADO, CDMUNICIPIO)
                                    SELECT
                                       :CDFORNECED,
                                       :CDPAIS,
                                       :SGESTADO,
                                       :CDMUNICIPIO
                                    FROM PARAMGERAL
                                        WHERE NOT EXISTS (SELECT 1
                                                              FROM FORNECMPEXTRA F
                                                              WHERE F.CDFORNECED  = :CDFORNECED
                                                                AND F.CDPAIS      = :CDPAIS
                                                                AND F.SGESTADO    = :SGESTADO
                                                                AND F.CDMUNICIPIO = :CDMUNICIPIO)";
                        $params = [
                            'CDFORNECED'    => $forneced,
                            'CDPAIS'        => $pais,
                            'SGESTADO'      => $estado,
                            'CDMUNICIPIO'   => $municipio
                        ];
                        $this->supUtilities->crud($query, $params);
                        $this->supUtilities->commit();
                    }
                    catch (Exception $e) {
                        throw new \Exception('Erro ao tentar salvar fornecedor(es)');
                        $this->supUtilities->rollback();
                    }
                }else{
                    try{
                        $this->supUtilities->iniciaTransacao();
                        $query = "DELETE FROM FORNECMPEXTRA
                                    WHERE CDFORNECED  = :CDFORNECED
                                        AND CDPAIS      = :CDPAIS
                                        AND SGESTADO    = :SGESTADO
                                        AND CDMUNICIPIO = :CDMUNICIPIO";
                        $params = [
                            'CDFORNECED'    => $forneced,
                            'CDPAIS'        => $pais,
                            'SGESTADO'      => $estado,
                            'CDMUNICIPIO'   => $municipio
                        ];
                        $this->supUtilities->crud($query, $params);
                        $this->supUtilities->commit();
                    }catch (Exception $e){
                        throw new \Exception('Erro ao tentar excluir fornecedor(es)');
                        $this->supUtilities->rollback();
                    }
                }
            }
        }
        
    }
    
    public function exportarFornecedores($row){
        $fornecedorOrigem = $row['FORNORIGEM'];
        $fornecedorDestino = $row['FORNDESTINO'];
        $municipioAtendido = $row['MUNICIPIOATENDIDO'];
        
        foreach($municipioAtendido as $key => $value){
            $municipio = $value['CDMUNICIPIO'];
            $estado = $value['SGESTADO'];
            $pais = $value['CDPAIS'];
            foreach($fornecedorDestino as $index => $data){
                $fornecedor = $data['CDFORNECED'];
                try {
                    $this->supUtilities->iniciaTransacao();
                    $query = "INSERT INTO FORNECMPEXTRA (CDFORNECED, CDPAIS, SGESTADO, CDMUNICIPIO)
                                SELECT 
                                   FORNECEDOR.CDFORNECED,
                                   FORNECMPEXTRA.CDPAIS,
                                   FORNECMPEXTRA.SGESTADO,
                                   FORNECMPEXTRA.CDMUNICIPIO
                                FROM
                                   FORNECMPEXTRA,
                                   FORNECEDOR
                                WHERE
                                   FORNECMPEXTRA.CDFORNECED = :P_CDFORNORIG AND
                                   ((FORNECEDOR.CDFORNECED = :P_CDFORNDEST) OR (:P_CDFORNDEST = 'T'))  AND
                                   ((FORNECMPEXTRA.CDPAIS = :P_PAIS) OR (:P_PAIS = 'T')) AND
                                   ((FORNECMPEXTRA.SGESTADO = :P_SGESTADO) OR (:P_SGESTADO = 'T')) AND
                                   ((FORNECMPEXTRA.CDMUNICIPIO = :P_CDMUNICIPIO) OR (:P_CDMUNICIPIO = 'T')) AND
                                   NOT EXISTS
                                    (SELECT
                                       1
                                     FROM
                                       FORNECMPEXTRA A
                                     WHERE A.CDPAIS = FORNECMPEXTRA.CDPAIS
                                       AND A.SGESTADO = FORNECMPEXTRA.SGESTADO
                                       AND A.CDMUNICIPIO = FORNECMPEXTRA.CDMUNICIPIO
                                       AND A.CDFORNECED = FORNECEDOR.CDFORNECED)";
                    $params = [
                        'P_CDFORNORIG'  => $fornecedorOrigem,
                        'P_CDFORNDEST'  => $fornecedor,
                        'P_PAIS' => $pais,
                        'P_SGESTADO' => $estado,
                        'P_CDMUNICIPIO' => $municipio,
                    ];
                    $this->supUtilities->crud($query, $params);
                    $this->supUtilities->commit();
                } catch (Exception $e) {
                    throw new \Exception('Erro ao tentar exportar fornecedor(es)');
                    $this->supUtilities->rollback();
                }
            }
        }
    }
    
    public function createListas($rows) {
        $NRLISTA_SGESTADO = null;
        $NRLISTA_CDMUNICIPIO = null;
        $NRLISTA_CDFORNECED = null;
        
        foreach($rows as $key => $value) {
            if($key === 'SGESTADO' && $value != 'T') {
                $SGESTADO = explode('_',$value);
                try{
                    $NRLISTA_SGESTADO = $this->supUtilities->insertLista($SGESTADO);
                }catch(\Exception $e){
                    $response->setError(new Error($e->getMessage(), 1));  
                }
            } else if($key === 'CDMUNICIPIO' && $value != 'T') {
                $CDMUNICIPIO = explode('_',$value);
                try{
                    $NRLISTA_CDMUNICIPIO = $this->supUtilities->insertLista($CDMUNICIPIO);
                }catch(\Exception $e){
                    $response->setError(new Error($e->getMessage(), 1));  
                }
            } else if($key === 'CDFORNECED' && $value != 'T') {
                $CDFORNECED = explode('_',$value);
                try{
                    $NRLISTA_CDFORNECED = $this->supUtilities->insertLista($CDFORNECED);
                }catch(\Exception $e){
                    $response->setError(new Error($e->getMessage(), 1));  
                }
            } 
            
            $result = [
                'estado' => $NRLISTA_SGESTADO,
                'municipio' => $NRLISTA_CDMUNICIPIO
            ];
        }
        return $result;
    }
    
    public function deleteListas($rows) {
        foreach($rows as $key => $value) {
            $ESTADO = explode('=', $value);
            if($key === 'ESTADO' && $value != 'T') {
                try{
                   $this->supUtilities->deleteLista($ESTADO[1]);
                }catch(\Exception $e){
                    $response->setError(new Error($e->getMessage(), 1));
                }
            } else if($key === 'MUNICIPIO' && $value != 'T') {
                $MUNICIPIO = explode('=',$value);
                try{
                    $this->supUtilities->deleteLista($MUNICIPIO[1]);
                }catch(\Exception $e){
                    $response->setError(new Error($e->getMessage(), 1));
                }
            } else if($key === 'CDFORNECED' && $value != 'T') {
                $CDFORNECED = explode('=',$value);
                try{
                    $this->supUtilities->deleteLista($CDFORNECED[1]);
                }catch(\Exception $e){
                    $response->setError(new Error($e->getMessage(), 1));
                }
            } 
        }
    }
    
}